import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'dart:typed_data';

abstract class ConnectionAction {
  Future onReceive(String localIp, int commandType, String readed);
  Future onConnectionClosed(String localIp);
  Future onConnectionError(String localIp, String err);
}
abstract class Connection {
  String id;
  ConnectionAction _doSomething;
  Future initialize(ConnectionAction doSomething);
  void send(int commandType, String text, String clientAddress, int port);
}
class IPConnection extends Connection {
  int localPort;
  RawDatagramSocket _socket;
  IPConnection(this.localPort);
  static Future<String> getIp() async {
    for (var interface in await NetworkInterface.list()) {
      for (var addr in interface.addresses) {
        return addr.address;
      }
    }
    return null;
  }
  int _commandType = -1;
  List<int> _buffer = [];
  static int startCommand = 255;
  static int endCommand = 254;
  Future initialize(ConnectionAction doSomething) async {
    this._doSomething = doSomething;
    id = await getIp();
    try {
      Future.wait([RawDatagramSocket.bind(new InternetAddress(id), localPort)]).then((values) {
        _socket = values[0];
        _socket.listen((RawSocketEvent e) {
          try {
            switch (e) {
              case RawSocketEvent.read:
                Datagram dg = _socket.receive();
                for(int i= 0; i<dg.data.length; i++) {
                  if (dg.data[i] == startCommand) {
                    _buffer = [];
                    _commandType = -1;
                  } else if (dg.data[i] == endCommand) {
                    _doSomething.onReceive(id, _commandType, new Utf8Codec().decode(_buffer));
                    _buffer = [];
                    _commandType = -1;
                  } else if (_commandType == -1) {
                    _commandType = dg.data[i];
                  } else
                    _buffer.add(dg.data[i]);
                }
                _socket.writeEventsEnabled = true;
                break;
              case RawSocketEvent.write:
                break;
              case RawSocketEvent.closed:
                _doSomething.onConnectionClosed(id);
            }
          } catch (ex) {
            _doSomething.onConnectionError(id, ex);
          }
        });
      });
    } catch (ex) {
      _doSomething.onConnectionError(id, ex);
    }
  }
  void send(int commandType, String text, String clientAddress, int port) {
    List<int> command = [];
    command.add(startCommand);
    command.add(commandType);
    var c = new Utf8Codec().encode(text);
    for(var i=0; i<c.length; i++)
      command.add(c[i]);
    command.add(endCommand);
    _socket.send(new Uint8List.fromList(command), new InternetAddress(clientAddress), port);
  }
}
